@@ -7,6 +7,7 @@
 
 import asyncio
 import contextlib
+import gc
 import ipaddress
 import os
 import pathlib
@@ -24,7 +25,7 @@ import weakref
 
 import asyncpg
 from asyncpg import _testbase as tb
-from asyncpg import connection
+from asyncpg import connection as pg_connection
 from asyncpg import connect_utils
 from asyncpg import cluster as pg_cluster
 from asyncpg import exceptions
@@ -71,6 +72,23 @@ def mock_dot_postgresql(*, ca=True, crl=False, client=False, protected=False):
             yield
 
 
+@contextlib.contextmanager
+def mock_no_home_dir():
+    with unittest.mock.patch(
+        'pathlib.Path.home', unittest.mock.Mock(side_effect=RuntimeError)
+    ):
+        yield
+
+
+@contextlib.contextmanager
+def mock_dev_null_home_dir():
+    with unittest.mock.patch(
+        'pathlib.Path.home',
+        unittest.mock.Mock(return_value=pathlib.Path('/dev/null')),
+    ):
+        yield
+
+
 class TestSettings(tb.ConnectedTestCase):
 
     async def test_get_settings_01(self):
@@ -109,6 +127,9 @@ class TestSettings(tb.ConnectedTestCase):
             self.assertEqual(expected, result)
 
 
+CORRECT_PASSWORD = 'correct\u1680password'
+
+
 class TestAuthentication(tb.ConnectedTestCase):
     def setUp(self):
         super().setUp()
@@ -119,9 +140,9 @@ class TestAuthentication(tb.ConnectedTestCase):
         methods = [
             ('trust', None),
             ('reject', None),
-            ('scram-sha-256', 'correctpassword'),
-            ('md5', 'correctpassword'),
-            ('password', 'correctpassword'),
+            ('scram-sha-256', CORRECT_PASSWORD),
+            ('md5', CORRECT_PASSWORD),
+            ('password', CORRECT_PASSWORD),
         ]
 
         self.cluster.reset_hba()
@@ -143,7 +164,7 @@ class TestAuthentication(tb.ConnectedTestCase):
             create_script.append(
                 'CREATE ROLE {}_user WITH LOGIN{};'.format(
                     username,
-                    ' PASSWORD {!r}'.format(password) if password else ''
+                    f' PASSWORD E{(password or "")!r}'
                 )
             )
 
@@ -233,7 +254,7 @@ class TestAuthentication(tb.ConnectedTestCase):
     async def test_auth_password_cleartext(self):
         conn = await self.connect(
             user='password_user',
-            password='correctpassword')
+            password=CORRECT_PASSWORD)
         await conn.close()
 
         with self.assertRaisesRegex(
@@ -245,7 +266,7 @@ class TestAuthentication(tb.ConnectedTestCase):
 
     async def test_auth_password_cleartext_callable(self):
         def get_correctpassword():
-            return 'correctpassword'
+            return CORRECT_PASSWORD
 
         def get_wrongpassword():
             return 'wrongpassword'
@@ -264,7 +285,7 @@ class TestAuthentication(tb.ConnectedTestCase):
 
     async def test_auth_password_cleartext_callable_coroutine(self):
         async def get_correctpassword():
-            return 'correctpassword'
+            return CORRECT_PASSWORD
 
         async def get_wrongpassword():
             return 'wrongpassword'
@@ -283,7 +304,7 @@ class TestAuthentication(tb.ConnectedTestCase):
 
     async def test_auth_password_cleartext_callable_awaitable(self):
         async def get_correctpassword():
-            return 'correctpassword'
+            return CORRECT_PASSWORD
 
         async def get_wrongpassword():
             return 'wrongpassword'
@@ -302,7 +323,7 @@ class TestAuthentication(tb.ConnectedTestCase):
 
     async def test_auth_password_md5(self):
         conn = await self.connect(
-            user='md5_user', password='correctpassword')
+            user='md5_user', password=CORRECT_PASSWORD)
         await conn.close()
 
         with self.assertRaisesRegex(
@@ -317,7 +338,7 @@ class TestAuthentication(tb.ConnectedTestCase):
             return
 
         conn = await self.connect(
-            user='scram_sha_256_user', password='correctpassword')
+            user='scram_sha_256_user', password=CORRECT_PASSWORD)
         await conn.close()
 
         with self.assertRaisesRegex(
@@ -354,7 +375,7 @@ class TestAuthentication(tb.ConnectedTestCase):
             await conn.close()
 
         alter_password = \
-            "ALTER ROLE scram_sha_256_user PASSWORD 'correctpassword';"
+            f"ALTER ROLE scram_sha_256_user PASSWORD E{CORRECT_PASSWORD!r};"
         await self.con.execute(alter_password)
         await self.con.execute("SET password_encryption = 'md5';")
 
@@ -364,7 +385,7 @@ class TestAuthentication(tb.ConnectedTestCase):
             exceptions.InternalClientError,
             ".*no md5.*",
         ):
-            await self.connect(user='md5_user', password='correctpassword')
+            await self.connect(user='md5_user', password=CORRECT_PASSWORD)
 
 
 class TestConnectParams(tb.TestCase):
@@ -384,7 +405,8 @@ class TestConnectParams(tb.TestCase):
                 'password': 'passw',
                 'database': 'testdb',
                 'ssl': True,
-                'sslmode': SSLMode.prefer})
+                'sslmode': SSLMode.prefer,
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -406,7 +428,8 @@ class TestConnectParams(tb.TestCase):
             'result': ([('host2', 456)], {
                 'user': 'user2',
                 'password': 'passw2',
-                'database': 'db2'})
+                'database': 'db2',
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -434,7 +457,8 @@ class TestConnectParams(tb.TestCase):
                 'password': 'passw2',
                 'database': 'db2',
                 'sslmode': SSLMode.disable,
-                'ssl': False})
+                'ssl': False,
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -455,7 +479,8 @@ class TestConnectParams(tb.TestCase):
                 'password': '123123',
                 'database': 'abcdef',
                 'ssl': True,
-                'sslmode': SSLMode.allow})
+                'sslmode': SSLMode.allow,
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -483,7 +508,8 @@ class TestConnectParams(tb.TestCase):
                 'password': 'passw2',
                 'database': 'db2',
                 'sslmode': SSLMode.disable,
-                'ssl': False})
+                'ssl': False,
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -504,7 +530,8 @@ class TestConnectParams(tb.TestCase):
                 'password': '123123',
                 'database': 'abcdef',
                 'ssl': True,
-                'sslmode': SSLMode.prefer})
+                'sslmode': SSLMode.prefer,
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -513,7 +540,8 @@ class TestConnectParams(tb.TestCase):
             'result': ([('localhost', 5555)], {
                 'user': 'user3',
                 'password': '123123',
-                'database': 'abcdef'})
+                'database': 'abcdef',
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -522,6 +550,7 @@ class TestConnectParams(tb.TestCase):
             'result': ([('host1', 5432), ('host2', 5432)], {
                 'database': 'db',
                 'user': 'user',
+                'target_session_attrs': 'any',
             })
         },
 
@@ -531,6 +560,43 @@ class TestConnectParams(tb.TestCase):
             'result': ([('host1', 1111), ('host2', 2222)], {
                 'database': 'db',
                 'user': 'user',
+                'target_session_attrs': 'any',
+            })
+        },
+
+        {
+            'name': 'target_session_attrs',
+            'dsn': 'postgresql://user@host1:1111,host2:2222/db'
+                   '?target_session_attrs=read-only',
+            'result': ([('host1', 1111), ('host2', 2222)], {
+                'database': 'db',
+                'user': 'user',
+                'target_session_attrs': 'read-only',
+            })
+        },
+
+        {
+            'name': 'target_session_attrs_2',
+            'dsn': 'postgresql://user@host1:1111,host2:2222/db'
+                   '?target_session_attrs=read-only',
+            'target_session_attrs': 'read-write',
+            'result': ([('host1', 1111), ('host2', 2222)], {
+                'database': 'db',
+                'user': 'user',
+                'target_session_attrs': 'read-write',
+            })
+        },
+
+        {
+            'name': 'target_session_attrs_3',
+            'dsn': 'postgresql://user@host1:1111,host2:2222/db',
+            'env': {
+                'PGTARGETSESSIONATTRS': 'read-only',
+            },
+            'result': ([('host1', 1111), ('host2', 2222)], {
+                'database': 'db',
+                'user': 'user',
+                'target_session_attrs': 'read-only',
             })
         },
 
@@ -540,6 +606,7 @@ class TestConnectParams(tb.TestCase):
             'result': ([('2001:db8::1234%eth0', 5432), ('::1', 5432)], {
                 'database': 'db',
                 'user': 'user',
+                'target_session_attrs': 'any',
             })
         },
 
@@ -549,6 +616,7 @@ class TestConnectParams(tb.TestCase):
             'result': ([('2001:db8::1234', 1111), ('::1', 2222)], {
                 'database': 'db',
                 'user': 'user',
+                'target_session_attrs': 'any',
             })
         },
 
@@ -558,6 +626,7 @@ class TestConnectParams(tb.TestCase):
             'result': ([('2001:db8::1234', 5432), ('::1', 5432)], {
                 'database': 'db',
                 'user': 'user',
+                'target_session_attrs': 'any',
             })
         },
 
@@ -572,6 +641,7 @@ class TestConnectParams(tb.TestCase):
             'result': ([('host1', 1111), ('host2', 2222)], {
                 'database': 'db',
                 'user': 'foo',
+                'target_session_attrs': 'any',
             })
         },
 
@@ -584,6 +654,7 @@ class TestConnectParams(tb.TestCase):
             'result': ([('host1', 1111), ('host2', 2222)], {
                 'database': 'db',
                 'user': 'foo',
+                'target_session_attrs': 'any',
             })
         },
 
@@ -597,6 +668,20 @@ class TestConnectParams(tb.TestCase):
             'result': ([('host1', 5432), ('host2', 5432)], {
                 'database': 'db',
                 'user': 'foo',
+                'target_session_attrs': 'any',
+            })
+        },
+        {
+            'name': 'params_multi_host_dsn_env_mix_tuple',
+            'env': {
+                'PGUSER': 'foo',
+            },
+            'dsn': 'postgresql:///db',
+            'host': ('host1', 'host2'),
+            'result': ([('host1', 5432), ('host2', 5432)], {
+                'database': 'db',
+                'user': 'foo',
+                'target_session_attrs': 'any',
             })
         },
 
@@ -616,7 +701,8 @@ class TestConnectParams(tb.TestCase):
                 'password': 'ask',
                 'database': 'db',
                 'ssl': True,
-                'sslmode': SSLMode.require})
+                'sslmode': SSLMode.require,
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -637,7 +723,8 @@ class TestConnectParams(tb.TestCase):
                 'password': 'ask',
                 'database': 'db',
                 'sslmode': SSLMode.verify_full,
-                'ssl': True})
+                'ssl': True,
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -645,7 +732,8 @@ class TestConnectParams(tb.TestCase):
             'dsn': 'postgresql:///dbname?host=/unix_sock/test&user=spam',
             'result': ([os.path.join('/unix_sock/test', '.s.PGSQL.5432')], {
                 'user': 'spam',
-                'database': 'dbname'})
+                'database': 'dbname',
+                'target_session_attrs': 'any'})
         },
 
         {
@@ -657,6 +745,7 @@ class TestConnectParams(tb.TestCase):
                     'user': 'us@r',
                     'password': 'p@ss',
                     'database': 'db',
+                    'target_session_attrs': 'any',
                 }
             )
         },
@@ -670,6 +759,7 @@ class TestConnectParams(tb.TestCase):
                     'user': 'user',
                     'password': 'p',
                     'database': 'db',
+                    'target_session_attrs': 'any',
                 }
             )
         },
@@ -682,6 +772,7 @@ class TestConnectParams(tb.TestCase):
                 {
                     'user': 'us@r',
                     'database': 'db',
+                    'target_session_attrs': 'any',
                 }
             )
         },
@@ -709,7 +800,8 @@ class TestConnectParams(tb.TestCase):
                     'user': 'user',
                     'database': 'user',
                     'sslmode': SSLMode.disable,
-                    'ssl': None
+                    'ssl': None,
+                    'target_session_attrs': 'any',
                 }
             )
         },
@@ -723,7 +815,8 @@ class TestConnectParams(tb.TestCase):
                     '.s.PGSQL.5432'
                 )], {
                     'user': 'spam',
-                    'database': 'db'
+                    'database': 'db',
+                    'target_session_attrs': 'any',
                 }
             )
         },
@@ -744,6 +837,7 @@ class TestConnectParams(tb.TestCase):
                     'database': 'db',
                     'ssl': True,
                     'sslmode': SSLMode.prefer,
+                    'target_session_attrs': 'any',
                 }
             )
         },
@@ -788,6 +882,7 @@ class TestConnectParams(tb.TestCase):
         database = testcase.get('database')
         sslmode = testcase.get('ssl')
         server_settings = testcase.get('server_settings')
+        target_session_attrs = testcase.get('target_session_attrs')
 
         expected = testcase.get('result')
         expected_error = testcase.get('error')
@@ -810,8 +905,9 @@ class TestConnectParams(tb.TestCase):
             addrs, params = connect_utils._parse_connect_dsn_and_args(
                 dsn=dsn, host=host, port=port, user=user, password=password,
                 passfile=passfile, database=database, ssl=sslmode,
-                direct_tls=False, connect_timeout=None,
-                server_settings=server_settings)
+                direct_tls=False,
+                server_settings=server_settings,
+                target_session_attrs=target_session_attrs)
 
             params = {
                 k: v for k, v in params._asdict().items()
@@ -872,7 +968,9 @@ class TestConnectParams(tb.TestCase):
                 'host': 'abc',
                 'result': (
                     [('abc', 5432)],
-                    {'user': '__test__', 'database': '__test__'}
+                    {'user': '__test__',
+                     'database': '__test__',
+                     'target_session_attrs': 'any'}
                 )
             })
 
@@ -910,6 +1008,7 @@ class TestConnectParams(tb.TestCase):
                         'password': 'password from pgpass for user@abc',
                         'user': 'user',
                         'database': 'db',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -926,6 +1025,7 @@ class TestConnectParams(tb.TestCase):
                         'password': 'password from pgpass for user@abc',
                         'user': 'user',
                         'database': 'db',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -940,6 +1040,7 @@ class TestConnectParams(tb.TestCase):
                         'password': 'password from pgpass for user@abc',
                         'user': 'user',
                         'database': 'db',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -955,6 +1056,7 @@ class TestConnectParams(tb.TestCase):
                         'password': 'password from pgpass for localhost',
                         'user': 'user',
                         'database': 'db',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -972,6 +1074,7 @@ class TestConnectParams(tb.TestCase):
                             'password': 'password from pgpass for localhost',
                             'user': 'user',
                             'database': 'db',
+                            'target_session_attrs': 'any',
                         }
                     )
                 })
@@ -989,6 +1092,7 @@ class TestConnectParams(tb.TestCase):
                         'password': 'password from pgpass for cde:5433',
                         'user': 'user',
                         'database': 'db',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -1005,6 +1109,7 @@ class TestConnectParams(tb.TestCase):
                         'password': 'password from pgpass for testuser',
                         'user': 'testuser',
                         'database': 'db',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -1021,6 +1126,7 @@ class TestConnectParams(tb.TestCase):
                         'password': 'password from pgpass for testdb',
                         'user': 'user',
                         'database': 'testdb',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -1037,6 +1143,7 @@ class TestConnectParams(tb.TestCase):
                         'password': 'password from pgpass with escapes',
                         'user': R'test\\',
                         'database': R'test\:db',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -1064,6 +1171,7 @@ class TestConnectParams(tb.TestCase):
                         {
                             'user': 'user',
                             'database': 'db',
+                            'target_session_attrs': 'any',
                         }
                     )
                 })
@@ -1084,6 +1192,7 @@ class TestConnectParams(tb.TestCase):
                         {
                             'user': 'user',
                             'database': 'db',
+                            'target_session_attrs': 'any',
                         }
                     )
                 })
@@ -1100,6 +1209,7 @@ class TestConnectParams(tb.TestCase):
                 {
                     'user': 'user',
                     'database': 'db',
+                    'target_session_attrs': 'any',
                 }
             )
         })
@@ -1120,6 +1230,7 @@ class TestConnectParams(tb.TestCase):
                     {
                         'user': 'user',
                         'database': 'db',
+                        'target_session_attrs': 'any',
                     }
                 )
             })
@@ -1142,6 +1253,7 @@ class TestConnectParams(tb.TestCase):
                             {
                                 'user': 'user',
                                 'database': 'db',
+                                'target_session_attrs': 'any',
                             }
                         )
                     })
@@ -1164,7 +1276,7 @@ class TestConnectParams(tb.TestCase):
 class TestConnection(tb.ConnectedTestCase):
 
     async def test_connection_isinstance(self):
-        self.assertTrue(isinstance(self.con, connection.Connection))
+        self.assertTrue(isinstance(self.con, pg_connection.Connection))
         self.assertTrue(isinstance(self.con, object))
         self.assertFalse(isinstance(self.con, list))
 
@@ -1257,6 +1369,46 @@ class TestConnection(tb.ConnectedTestCase):
             user=conn_spec.get('user'))
         await con.close()
 
+    @unittest.skipIf(os.environ.get('PGHOST'), 'unmanaged cluster')
+    async def test_connection_no_home_dir(self):
+        with mock_no_home_dir():
+            con = await self.connect(
+                dsn='postgresql://foo/',
+                user='postgres',
+                database='postgres',
+                host='localhost')
+            await con.fetchval('SELECT 42')
+            await con.close()
+
+        with mock_dev_null_home_dir():
+            con = await self.connect(
+                dsn='postgresql://foo/',
+                user='postgres',
+                database='postgres',
+                host='localhost')
+            await con.fetchval('SELECT 42')
+            await con.close()
+
+        with self.assertRaisesRegex(
+            exceptions.ClientConfigurationError,
+            r'root certificate file "~/\.postgresql/root\.crt" does not exist'
+        ):
+            with mock_no_home_dir():
+                await self.connect(
+                    host='localhost',
+                    user='ssl_user',
+                    ssl='verify-full')
+
+        with self.assertRaisesRegex(
+            exceptions.ClientConfigurationError,
+            r'root certificate file ".*" does not exist'
+        ):
+            with mock_dev_null_home_dir():
+                await self.connect(
+                    host='localhost',
+                    user='ssl_user',
+                    ssl='verify-full')
+
 
 class BaseTestSSLConnection(tb.ConnectedTestCase):
     @classmethod
@@ -1497,13 +1649,14 @@ class TestSSLConnection(BaseTestSSLConnection):
                             '&ssl_min_protocol_version=TLSv1.1'
                             '&ssl_max_protocol_version=TLSv1.1'
                     )
-                with self.assertRaisesRegex(ssl.SSLError, 'no protocols'):
-                    await self.connect(
-                        dsn='postgresql://ssl_user@localhost/postgres'
-                            '?sslmode=require'
-                            '&ssl_min_protocol_version=TLSv1.2'
-                            '&ssl_max_protocol_version=TLSv1.1'
-                    )
+                if not ssl.OPENSSL_VERSION.startswith('LibreSSL'):
+                    with self.assertRaisesRegex(ssl.SSLError, 'no protocols'):
+                        await self.connect(
+                            dsn='postgresql://ssl_user@localhost/postgres'
+                                '?sslmode=require'
+                                '&ssl_min_protocol_version=TLSv1.2'
+                                '&ssl_max_protocol_version=TLSv1.1'
+                        )
                 con = await self.connect(
                     dsn='postgresql://ssl_user@localhost/postgres'
                         '?sslmode=require'
@@ -1707,14 +1860,27 @@ class TestNoSSLConnection(BaseTestSSLConnection):
 class TestConnectionGC(tb.ClusterTestCase):
 
     async def _run_no_explicit_close_test(self):
-        con = await self.connect()
-        await con.fetchval("select 123")
-        proto = con._protocol
-        conref = weakref.ref(con)
-        del con
+        gc_was_enabled = gc.isenabled()
+        gc.disable()
+        try:
+            con = await self.connect()
+            await con.fetchval("select 123")
+            proto = con._protocol
+            conref = weakref.ref(con)
+            del con
 
-        self.assertIsNone(conref())
-        self.assertTrue(proto.is_closed())
+            self.assertIsNone(conref())
+            self.assertTrue(proto.is_closed())
+
+            # tick event loop; asyncio.selector_events._SelectorSocketTransport
+            # needs a chance to close itself and remove its reference to proto
+            await asyncio.sleep(0)
+            protoref = weakref.ref(proto)
+            del proto
+            self.assertIsNone(protoref())
+        finally:
+            if gc_was_enabled:
+                gc.enable()
 
     async def test_no_explicit_close_no_debug(self):
         olddebug = self.loop.get_debug()
@@ -1735,8 +1901,96 @@ class TestConnectionGC(tb.ClusterTestCase):
                                        r'unclosed connection') as rw:
                 await self._run_no_explicit_close_test()
 
-            msg = rw.warning.args[0]
+            msg = " ".join(rw.warning.args)
             self.assertIn(' created at:\n', msg)
             self.assertIn('in test_no_explicit_close_with_debug', msg)
         finally:
             self.loop.set_debug(olddebug)
+
+
+class TestConnectionAttributes(tb.HotStandbyTestCase):
+
+    async def _run_connection_test(
+        self, connect, target_attribute, expected_port
+    ):
+        conn = await connect(target_session_attrs=target_attribute)
+        self.assertTrue(_get_connected_host(conn).endswith(expected_port))
+        await conn.close()
+
+    async def test_target_server_attribute_port(self):
+        master_port = self.master_cluster.get_connection_spec()['port']
+        standby_port = self.standby_cluster.get_connection_spec()['port']
+        tests = [
+            (self.connect_primary, 'primary', master_port),
+            (self.connect_standby, 'standby', standby_port),
+        ]
+
+        for connect, target_attr, expected_port in tests:
+            await self._run_connection_test(
+                connect, target_attr, expected_port
+            )
+        if self.master_cluster.get_pg_version()[0] < 14:
+            self.skipTest("PostgreSQL<14 does not support these features")
+        tests = [
+            (self.connect_primary, 'read-write', master_port),
+            (self.connect_standby, 'read-only', standby_port),
+        ]
+
+        for connect, target_attr, expected_port in tests:
+            await self._run_connection_test(
+                connect, target_attr, expected_port
+            )
+
+    async def test_target_attribute_not_matched(self):
+        tests = [
+            (self.connect_standby, 'primary'),
+            (self.connect_primary, 'standby'),
+        ]
+
+        for connect, target_attr in tests:
+            with self.assertRaises(exceptions.TargetServerAttributeNotMatched):
+                await connect(target_session_attrs=target_attr)
+
+        if self.master_cluster.get_pg_version()[0] < 14:
+            self.skipTest("PostgreSQL<14 does not support these features")
+        tests = [
+            (self.connect_standby, 'read-write'),
+            (self.connect_primary, 'read-only'),
+        ]
+
+        for connect, target_attr in tests:
+            with self.assertRaises(exceptions.TargetServerAttributeNotMatched):
+                await connect(target_session_attrs=target_attr)
+
+    async def test_prefer_standby_when_standby_is_up(self):
+        con = await self.connect(target_session_attrs='prefer-standby')
+        standby_port = self.standby_cluster.get_connection_spec()['port']
+        connected_host = _get_connected_host(con)
+        self.assertTrue(connected_host.endswith(standby_port))
+        await con.close()
+
+    async def test_prefer_standby_picks_master_when_standby_is_down(self):
+        primary_spec = self.get_cluster_connection_spec(self.master_cluster)
+        connection_spec = {
+            'host': [
+                primary_spec['host'],
+                'unlocalhost',
+            ],
+            'port': [primary_spec['port'], 15345],
+            'database': primary_spec['database'],
+            'user': primary_spec['user'],
+            'target_session_attrs': 'prefer-standby'
+        }
+
+        con = await self.connect(**connection_spec)
+        master_port = self.master_cluster.get_connection_spec()['port']
+        connected_host = _get_connected_host(con)
+        self.assertTrue(connected_host.endswith(master_port))
+        await con.close()
+
+
+def _get_connected_host(con):
+    peername = con._transport.get_extra_info('peername')
+    if isinstance(peername, tuple):
+        peername = "".join((str(s) for s in peername if s))
+    return peername
